a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
a {
	content: var(TEST)
}
